package csc.jms.cluster.ejb;

import java.util.HashMap;
import java.util.Properties;

import org.jboss.ejb3.annotation.ResourceAdapter;
import javax.ejb.ActivationConfigProperty;
import javax.ejb.MessageDriven;
import javax.ejb.TransactionAttribute;
import javax.ejb.TransactionAttributeType;
import javax.ejb.TransactionManagement;
import javax.ejb.TransactionManagementType;
import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Queue;
import javax.jms.Session;
import javax.naming.InitialContext;

@MessageDriven(activationConfig = {
		@ActivationConfigProperty(propertyName = "destinationType", propertyValue = "javax.jms.Queue"),
		@ActivationConfigProperty(propertyName = "destination", propertyValue = "queue/requestQ"),
		@ActivationConfigProperty(propertyName = "acknowledgeMode", propertyValue = "Auto-acknowledge") })
@ResourceAdapter("hornetq-ra.rar")

@TransactionManagement(value = TransactionManagementType.CONTAINER)
@TransactionAttribute(value = TransactionAttributeType.REQUIRED)
public class MDB1 implements 	MessageListener {

	private static final int MILLI_SECONDS = 1000;

	//@Resource(mappedName = "java:RemoteJmsXA")
	private ConnectionFactory connectionFactory;

	public MDB1() {
	}

	public void onMessage(Message message) {
		try {
			System.out.println(" Message consumed by mdb ");
			delay(15);
			sendResponse("Message " + patentDocument.getIANumber() + "received");
		} catch (JMSException e) {
			e.printStackTrace();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	private void sendResponse(String message) {
		System.out.println("0. About to send the response message : "
				+ connectionFactory);
		Connection conn = null;
		try {
			//conn = connectionFactory.createConnection();
			conn = getConnectionFactory().createConnection(); 
		
			System.out.println("1. created connection : " + conn);

			Session sess = conn.createSession(false, Session.AUTO_ACKNOWLEDGE);
			System.out.println("2. created session");

			Queue destQueueA = sess.createQueue("responseQ");
			System.out.println("3. created queue");

			MessageProducer prodA = sess.createProducer(destQueueA);
			System.out.println("4. created producer");

			prodA.send(sess.createTextMessage(message));
			System.out.println("Sent message \"" + message + "\" to responseQ");

		} catch (JMSException e) {
			e.printStackTrace();
		} catch (Exception e) {
				e.printStackTrace();
		} finally {
			// Step 7. Close the connection. (Since this is a JCA connection,
			// this will just place the connection back to a connection pool)
			try {
				if (conn != null) {
					conn.close();
				}
			} catch (JMSException e) {
				e.printStackTrace();
			}
		}
	}

	public ConnectionFactory getConnectionFactory() throws Exception {
		InitialContext initialContext = getContext();
		connectionFactory = (ConnectionFactory) initialContext
				.lookup("XAConnectionFactory");
		return connectionFactory;
	}

	protected static InitialContext getContext()
			throws Exception {
		Properties props = new Properties();

		HashMap map = new HashMap();
		map.put("java.naming.factory.initial",
				"org.jnp.interfaces.NamingContextFactory");
		map.put("java.naming.provider.url", "live:1199,backup:1299");
		map.put("java.naming.factory.url.pkgs",
				"org.jboss.naming:org.jnp.interfaces");
		map.put("jnp.partitionName", "QueuePartition"); 
		props.putAll(map);
		return new InitialContext(props);
	}

	private void delay(int seconds) {
		try {
			Thread.sleep(seconds * MILLI_SECONDS);
		} catch (InterruptedException e) {
			e.printStackTrace();
		}
	}
}